/*
 * @Author: 634665781 634665781@qq.com
 * @Date: 2023-12-06 09:16:37
 * @LastEditors: 634665781 634665781@qq.com
 * @LastEditTime: 2023-12-11 17:07:06
 * @FilePath: \xform\src\core\widgets\text\NumberInput\index.js
 * @Description: 这是默认设置,请设置`customMade`, 打开koroFileHeader查看配置 进行设置: https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 */
import React, { useEffect, useMemo, useState, useCallback } from 'react'
import { InputNumber, Input } from 'antd'
import styles from './index.less'

function formatMoney(number, places, symbol, thousand, decimal) {
  number = number || 0;
  places = !isNaN(places = Math.abs(places)) ? places : 2;
  symbol = symbol !== undefined ? symbol : "￥";
  thousand = thousand || ",";
  decimal = decimal || ".";
  var negative = number < 0 ? "-" : "",
    i = parseInt(number = Math.abs(+number || 0).toFixed(places), 10) + "",
    j = (j = i.length) > 3 ? j % 3 : 0;
  return symbol + negative + (j ? i.substr(0, j) + thousand : "") + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + thousand) + (places ? decimal + Math.abs(number - i).toFixed(places).slice(2) : "");
}

const debounce = (fn) => {
  let t = null
  return function (e) {
    const context = this
    const args = arguments
    if (t) {
      clearTimeout(t)
    }
    t = setTimeout(function () {
      fn.call(context, args)
    }, 200)
  }
}
const NumberInput = (props) => {

  const { value, onChange, schema, addons } = props
  const { disabled, presetValue, placeholder, prefix, formatter, decimalDigits, isStoreFormatter, addonBefore, addonAfter, min, max } = schema
  
  //新增最大最小值校验
  useEffect(()=>{
    if (addons && (min || max) && value) {
      let getValue = async (params) => {
        const [rule, value, callback] = params
        if(min){
          if(Number(value) < Number(min)){
            callback(new Error(`最小值不能小于${min}`))
          }else{
            callback()
          }
        }

        if(max){
          if(Number(value) > Number(max)){
            callback(new Error(`最大值不能大于${max}`))
          }else{
            callback()
          }
        }
      }
      let uniqueRules = [
        {
          validator: debounce(getValue)
        }
      ]
      addons.setSchemaByPath(addons.dataPath, { ...schema, rules: uniqueRules })
    }
  }, [value])

  useEffect(() => {
    if (addons) {
      addons.setValueByPath(addons.dataPath, presetValue)
    } else {
      onChange(presetValue)
    }
    
  }, [presetValue])

  const numberChange = (value) => {
    if (addons) {
      if (value !== null) {
        onChange(`${value}`)
      } else {
        onChange('')
      }

    
    }
  }

  const formatterFn = (value) => {
    if (value) {
      if (formatter === '${百分比}') {
        return `${value}%`
      } else if (formatter === '${货币}') {
        return formatMoney(Number(value), Number(decimalDigits))
      } else if (formatter === '${整数}') {
        return `${parseInt(value)}`
      } else if (formatter === '${正整数}') {
        return `${parseInt(value)}`
      } else if (['${小数}', '${正数}'].includes(formatter)) {
        return value
        // return Number(value).toFixed(Number(decimalDigits))
      } else if (formatter === '${科学计数法}') {
        return Number(value).toExponential()
      } else if (formatter) {
        return `${value}${formatter}`
      }
    }
    return value
  }

  const obj = ['${小数}', '${正数}'].includes(formatter) ? { precision: decimalDigits } : { formatter: formatterFn }

  return (
    <div className={styles.inputNumber} isdisabled={JSON.stringify(disabled)}>
      <InputNumber
        {...obj}
        min={['${正整数}', '${正数}'].includes(formatter) ? 0 : (min || Number.MIN_SAFE_INTEGER)}
        max={max || Number.MAX_SAFE_INTEGER}
        step={(formatter === '${百分比}' && isStoreFormatter) ? 0.01 : 1}
        addonBefore={addonBefore}
        addonAfter={addonAfter}
        prefix={prefix}
        placeholder={disabled ? (placeholder || '') : (placeholder || '请输入内容')}
        disabled={disabled}
        value={value === '' ? null : Number(value)}
        stringMode={true}
        onChange={numberChange}
        style={{ width: '100%' }}
      />
    </div>
  )

}

export default NumberInput